package mySweetFactory.measurement;



import javax.realtime.ImmortalMemory;
import javax.realtime.MemoryArea;
import javax.realtime.WaitFreeReadQueue;

import util.ContextChecker;




/**
 * Object pool for measurements.
 * 
 * 
 * In our design, NHRTs will be getting new measurements and a heap-based
 * thread will be putting them back. We can't share a monitor (lock) between NHRTs
 * and heap-based threads because then priority inheritance could place heap-based
 * thread at the same priority as the NHRT and in doing so force the garbage collector
 * to a priority higher than the NHRTs (totally defeating the point).
 * 
 * We are going to use a WaitFreeReadQueue to decouple the no-heap and heap-based
 * system components. Since we may have multiple NHRTs requiring measurements, we have
 * to synchronize our access to the WaitFreeReadQueue.read() method. This is acceptable
 * as long as we use a different lock - one only contended by NHRTs.
 * 
 * If there were many production lines contention on the fetching lock might become
 * an issue; in this scenario we could create multiple pools to share the load.
 * 
 * WaitFreeReadQueues have to be created with a fixed size which means the pool has to be
 * of fixed size, so it is possible that we could run out (if they all ended up sitting
 * with the audit thread). This would be the result of misconfiguration (too much load)
 * and we just have to deal with it as best we can.
 * 
 * 
 * 
 *
 */


public class MeasurementManager implements IMeasurementManager{

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;
	
	
	/**
	 * The size of the pool - the number of measurements in the system.
	 */
	public static int POOL_SIZE = 400;
	
	
	/**
	 * The pool - set to not-block on pool empty (would rather defeat the point in this context)
	 */
	private final WaitFreeReadQueue poolQueue = new WaitFreeReadQueue(POOL_SIZE,false);
	
	
	
	
	public MeasurementManager()
	{
		ContextChecker.checkContext(IMMORTAL_CONTEXT);
		
		//System.out.println("MeasurementManager : default constructor");
		
		for(int i=0;i!=POOL_SIZE;i++)
		{
			Measurement newMeasurement = new Measurement();
			
			returnMeasurement(newMeasurement);
		}
		
	}
	
	
	
	
	/**
	 * 
	 */
	public Measurement getMeasurement() {
		Measurement measurement = (Measurement) poolQueue.read();
		 
		if (measurement == null) {
			//TODO: put into ScopedMemory
			System.out.println("no measurements left");
			throw new RuntimeException("Pool unexpectedly full");
		}
		
		return measurement;
	}

	
	
	
	
	/**
	 * Checks a measurement back into the pool.
	 * 
	 * @param m Measurement to be checked in.
	 */
	public void returnMeasurement(Measurement m)
	{
		ContextChecker.checkContext(IMMORTAL_CONTEXT);
		
		/*This shouldn't happen unless someone was creating more measurements*/
		if(poolQueue.isFull())
		{
			throw new RuntimeException("Pool unexpectedly full");
		}

	
		
		try {
			poolQueue.write(m);
		} catch (InterruptedException e) {
			throw new RuntimeException("Unexpected interruption.",e);
		}
	}
	
	
	

}
