/*
 * DeveloperWorks RTJ Article 5 SweetFactory Demo
 * (C) Copyright IBM Corp. 2007. All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 */

package util;

import javax.realtime.HeapMemory;
import javax.realtime.ImmortalMemory;
import javax.realtime.NoHeapRealtimeThread;
import javax.realtime.RealtimeThread;
import javax.realtime.ScopedMemory;




/**
 * Utility class for checking context at any particular point.
 * 
 * @author Andrew Hall - andhall@uk.ibm.com
 * 
 */
public class ContextChecker {
	
	/**
	 * Constant representing working in the heap.
	 */
	public static final int HEAP_CONTEXT = 1;

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;

	/**
	 * Constant representing working in a scope.
	 */
	public static final int SCOPE_CONTEXT = 4;

	/**
	 * Constant representing working on a java.lang.Thread
	 */
	public static final int SE_THREAD = 8;

	/**
	 * Constant representing working on a javax.realtime.RealtimeThread
	 * or a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int RT_THREAD = 16;

	/**
	 * Constant representing working on a javax.realtime.NoHeapRealtimeThread
	 */
	public static final int NHRT_THREAD = 32;

	/**
	 * Checks it is called in the desired context.
	 * 
	 * Usage:
	 * 
	 * ContextChecker.checkContext(ContextChecker.SE_THREAD | ContextChecker.HEAP_CONTEXT);
	 * 
	 * To check you're on a java.lang.Thread in the heap.
	 * 
	 * @param flags Set of flags representing your desired state
	 * @throws RuntimeException is the current thread state is not what is 
	 * described by the flags.
	 */
	public static boolean checkContext(int flags) {
		if ((flags & HEAP_CONTEXT) > 0) {
			if (checkHeapContext())
				return true;
			else 
				return false;
		}

		if ((flags & IMMORTAL_CONTEXT) > 0) {
			if (checkImmortalContext())
			    return true;
			else 
				return false;
		}

		if ((flags & SCOPE_CONTEXT) > 0) {
			if (checkScopeContext())
				return true;
			else 
				return false;
		}

		if ((flags & SE_THREAD) > 0) {
			if (checkSEThread())
				return true;
			else 
				return false;
		}

		if ((flags & RT_THREAD) > 0) {
			if (checkRTThread())
				return true;
			else 
				return false;
		}

		if ((flags & NHRT_THREAD) > 0) {
			if (checkNHRTThread())
				return true;
			else 
				return false;
		}
		
		return false;
	}

	private static boolean checkNHRTThread() {
		if (!(Thread.currentThread() instanceof NoHeapRealtimeThread)) {
			throw new RuntimeException(
					"Error: should be on an NHRT Thread, but we're on a "
							+ Thread.currentThread().getClass().getName());
		}
		return true;
	}

	private static boolean checkRTThread() {
		if (!(Thread.currentThread() instanceof RealtimeThread)) {
			throw new RuntimeException(
					"Error: should be on an RT Thread, but we're on a "
							+ Thread.currentThread().getClass().getName());
		}
		
		return true;
	}

	private static boolean checkSEThread() {
		if (Thread.currentThread() instanceof RealtimeThread) {
			throw new RuntimeException(
					"Error: should be on an SE Thread, but we're on a "
							+ Thread.currentThread().getClass().getName());
		}
		
		return true;
	}

	private static boolean checkScopeContext() {
		if (!(RealtimeThread.getCurrentMemoryArea() instanceof ScopedMemory)) {
			throw new RuntimeException(
					"Error: should be in scoped memory was actually in "
							+ RealtimeThread.getCurrentMemoryArea());
		}
		
		return true;
	}

	private static boolean checkImmortalContext() {
		if (RealtimeThread.getCurrentMemoryArea() != ImmortalMemory.instance()) {
			throw new RuntimeException(
					"Error: should be in immortal memory was actually in "
							+ RealtimeThread.getCurrentMemoryArea());
		}
		
		return true;
	}

	private static boolean checkHeapContext() {
		if (RealtimeThread.getCurrentMemoryArea() != HeapMemory.instance()) {
			throw new RuntimeException(
					"Error: should be in heap memory was actually in "
							+ RealtimeThread.getCurrentMemoryArea());
		}
		
		return true;
	}
}
