package mySweetFactory.measurement;



import javax.realtime.LTMemory;
import javax.realtime.WaitFreeWriteQueue;

import mySweetFactory.util.ContextChecker;
import mySweetFactory.util.MemoryUsagePrint;




/**
 * Object pool for measurements.
 * 
 * 
 * In our design, NHRTs will be getting new measurements and a heap-based
 * thread will be putting them back. We can't share a monitor (lock) between NHRTs
 * and heap-based threads because then priority inheritance could place heap-based
 * thread at the same priority as the NHRT and in doing so force the garbage collector
 * to a priority higher than the NHRTs (totally defeating the point).
 * 
 * We are going to use a WaitFreeReadQueue to decouple the no-heap and heap-based
 * system components. Since we may have multiple NHRTs requiring measurements, we have
 * to synchronize our access to the WaitFreeReadQueue.read() method. This is acceptable
 * as long as we use a different lock - one only contended by NHRTs.
 * 
 * If there were many production lines contention on the fetching lock might become
 * an issue; in this scenario we could create multiple pools to share the load.
 * 
 * WaitFreeReadQueues have to be created with a fixed size which means the pool has to be
 * of fixed size, so it is possible that we could run out (if they all ended up sitting
 * with the audit thread). This would be the result of misconfiguration (too much load)
 * and we just have to deal with it as best we can.
 * 
 * 
 * 
 *
 */














public class MeasurementManager implements IMeasurementManager{

	/**
	 * Constant representing working in immortal.
	 */
	public static final int IMMORTAL_CONTEXT = 2;
	
	
	/**
	 * The size of the pool - the number of measurements in the system.
	 */
	public static int POOL_SIZE = 1;
	
	
	/**
	 * The pool - set to not-block on pool empty (would rather defeat the point in this context)
	 */
	private final WaitFreeWriteQueue poolQueue = new WaitFreeWriteQueue(POOL_SIZE);
	
	
	/**
	 * 
	 * 
	 * Benchmarks - helper classes for storing benchmarks
	 * 
	 */
	private LTMemory scopedMemory; 	
	
	
	
	
	
	
		
	
	/**
	 * Runnable for storing the benchmark into the file
	 * 
	 */
	Benchmark benchmark;
	
	/**
	 * for retrieving measurements 
	 * 
	 */
	Measurement currentMeasurement = null;
	
	
	
	/**
	 * 
	 *  Runnable for printing memory usage 
	 * 
	 */
	MemoryUsagePrint printUsage;
	
	/**
	 * 
	 * Runnable for getting the current Time
	 * 
	 * 
	 */
	GetTimeRunnable getTimeRunnable;
	
	
	String measurementValue = "measurement value";
	
	/**
	 * 
	 * 
	 * 
	 */
	public MeasurementManager()
	{
		
		
		/**
		 * 
		 * Allocating the measurements
		 * 
		 * 
		 */
		ContextChecker.checkContext(IMMORTAL_CONTEXT);
		for(int i=0;i!=POOL_SIZE;i++)
		{
			
			Measurement newMeasurement = new Measurement(measurementValue);
			poolQueue.write(newMeasurement);
		}
		
	
		
		
		/**
		 * 
		 * BENCHMARK setting
		 * 
		 * initializing the logic for gathering the benchmark results
		 * 
		 * 
		 */
		/**
		 * scoped memory where the benchmark threads run
		 */
		scopedMemory = new LTMemory(8*1024); 	
		
		
		benchmark = new Benchmark(); 
		
		
		
		
		
		/**
		 * 
		 * 
		 * 
		 */
		printUsage = new MemoryUsagePrint();
		printUsage.setCallerName("[MemoryManager]");
	
		
		getTimeRunnable = new GetTimeRunnable();
		
	}
	
	
	
	
	/**
	 * 
	 * 
	 * 
	 * 
	 * 
	 * 
	 */
	public Measurement getMeasurement() {
		//memConsumed();
		
		
		try {
			currentMeasurement = (Measurement) poolQueue.read();
		
			
			
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		 
		
		
		/**
		 * TODO:saving benchmark data
		 */
		//startOut.inputData = time.toString();
		//scopedMemory.enter(startOut);
		
		
		getStartTime();
		//memConsumed();
		
		return currentMeasurement;
	}

	
	
	
	
	/**
	 * Checks a measurement back into the pool.
	 * 
	 * @param m Measurement to be checked in.
	 */
	public void returnMeasurement(Measurement m)
	{
		ContextChecker.checkContext(IMMORTAL_CONTEXT);
		//memConsumed();
		
		getEndTime();
		
		//report Benchmark Iteration
		scopedMemory.enter(benchmark);
		
		//and return the measurement
		poolQueue.write(m);
		
		
		//memConsumed();
	}
	
	
	
	
	
	/**
   * 
   * 
   */
	private void memConsumed() {
		scopedMemory.enter(printUsage);
	}
	
	
	/**
	 * 
	 */
	private void getStartTime() {
		scopedMemory.enter(getTimeRunnable);
		benchmark.iterationStart.set(getTimeRunnable.result.getMilliseconds(),getTimeRunnable.result.getNanoseconds());
		
		//System.out.println("---------start-->"+benchmark.iterationStart);
		
	}
	
	
	/**
	 * 
	 */
	private void getEndTime() {
		scopedMemory.enter(getTimeRunnable);
		//benchmark.iterationEnd = getTimeRunnable.result;
		benchmark.iterationEnd.set(getTimeRunnable.result.getMilliseconds(),getTimeRunnable.result.getNanoseconds());
		
		//System.out.println("------------end--->" + benchmark.iterationStart);
	}

}
